Attribute VB_Name = "modBattlefield"
'
'  Wall of the Words is Copyright (c) 2008 Robson
'
'  This program is free software; you can redistribute it and/or modify
'  it under the terms of the GNU General Public License as published by
'  the Free Software Foundation; either version 2 of the License, or
'  (at your option) any later version.
'
'  This program is distributed in the hope that it will be useful, but
'  WITHOUT ANY WARRANTY; without even the implied warranty of
'  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
'  General Public License for more details.
'
'  The license can be viewed at http://www.gnu.org/licenses/gpl.html
'
' all variables are declared
Option Explicit

Private Type SQUARE
    blnGrass As Boolean
    blnEnemy As Boolean
    blnDefenceWall As Boolean
    blnDefenceWallTemporary As Boolean
    blnCastleWall As Boolean
    blnCastleMud As Boolean
    blnTree As Boolean
    blnWater As Boolean
    intHitPoints As Integer
    strLetter As String
    blnAffected As Boolean
    blnHitMinor As Boolean
    blnHitMajor As Boolean
End Type

Public arrBattlefield() As SQUARE
Public arrLetters() As String
Public blnLettersInUse() As Boolean
Public blnInGame As Boolean

Public Sub MakeLevel()
    ReDim arrBattlefield(bytBattlefieldWidth, bytBattlefieldHeight)
    Dim X As Integer, Y As Integer
    ' grass
    For X = 0 To UBound(arrBattlefield, 1)
        For Y = 0 To UBound(arrBattlefield, 2) - 3
            arrBattlefield(X, Y).blnGrass = True
        Next
    Next
    ' castlewall
    For X = 0 To UBound(arrBattlefield, 1)
        arrBattlefield(X, UBound(arrBattlefield, 2) - 2).blnCastleWall = True
    Next
    ' castle mud
    For X = 0 To UBound(arrBattlefield, 1)
        For Y = UBound(arrBattlefield, 2) - 2 To UBound(arrBattlefield, 2)
            arrBattlefield(X, Y).blnCastleMud = True
        Next
    Next
    SetBattlefieldSize UBound(arrBattlefield, 1), UBound(arrBattlefield, 2)
    AddInitialWords
    MakeInitialLetters
    AddObstacles
    EnemySetup
    For X = 0 To frmMain.picLetter.UBound
        frmMain.picLetter(X).DragMode = 1
    Next
    blnInGame = True
End Sub

Private Sub AddObstacles()
    Dim X As Byte, Y As Byte, A As Byte, B As Byte
    B = RNG(1, 4)
    Do
        X = RNG(1, UBound(arrBattlefield, 1))
        Y = RNG(1, UBound(arrBattlefield, 2))
        With arrBattlefield(X, Y)
            If .blnGrass And Not .blnDefenceWall Then
                .blnGrass = False
                .blnWater = True
                A = A + 1
            End If
        End With
    Loop Until A >= B
    A = 0
    B = RNG(1, 4)
    Do
        X = RNG(1, UBound(arrBattlefield, 1))
        Y = RNG(1, UBound(arrBattlefield, 2))
        With arrBattlefield(X, Y)
            If .blnGrass And Not .blnDefenceWall Then
                .blnGrass = False
                .blnTree = True
                A = A + 1
            End If
        End With
    Loop Until A >= B
End Sub

Private Sub AddInitialWords()
    Dim strWord As String
    Dim X As Byte, Y As Byte
    Dim A As Byte, B As Byte
    X = 0
    Y = RNG(bytBattlefieldHeight - 8, bytBattlefieldHeight - 5)
    Do
        If strWord = "" Then
            strWord = FindWord(Chr(Asc("a") + RNG(0, 25)) & "*", RNG(3, 5))
        Else
            strWord = FindWord(arrBattlefield(X, Y).strLetter & "*", RNG(3, 5))
        End If
        For A = 1 To Len(strWord)
            With arrBattlefield((A + X) - 1, Y)
                .strLetter = Mid(strWord, A, 1)
                .intHitPoints = bytInitialWallPoints 'bytLetterPoints(Asc(.strLetter) - Asc("a"))
                .blnDefenceWall = True
            End With
        Next
        X = X + (A - 2)
        If Y > bytBattlefieldHeight - 7 Then
            strWord = FindWord("*" & Right(strWord, 1), RNG(3, 4))
            For A = 1 To Len(strWord)
                With arrBattlefield(X, (Y - Len(strWord)) + A)
                    .strLetter = Mid(strWord, A, 1)
                    .intHitPoints = bytInitialWallPoints 'bytLetterPoints(Asc(.strLetter) - Asc("a"))
                    .blnDefenceWall = True
                End With
            Next
            Y = Y - (A - 2)
        Else
            strWord = FindWord(Right(strWord, 1) & "*", RNG(3, 4))
            For A = 1 To Len(strWord)
                With arrBattlefield(X, (Y + A) - 1)
                    .strLetter = Mid(strWord, A, 1)
                    .intHitPoints = bytInitialWallPoints 'bytLetterPoints(Asc(.strLetter) - Asc("a"))
                    .blnDefenceWall = True
                End With
            Next
            Y = Y + (A - 2)
        End If
    Loop Until X > UBound(arrBattlefield, 1) - 7
    strWord = FindWord(arrBattlefield(X, Y).strLetter & "*", UBound(arrBattlefield, 1) - X)
    For A = 1 To Len(strWord)
        With arrBattlefield((A + X) - 1, Y)
            .strLetter = Mid(strWord, A, 1)
            .intHitPoints = bytInitialWallPoints ' bytLetterPoints(Asc(.strLetter) - Asc("a"))
            .blnDefenceWall = True
        End With
    Next
End Sub

Private Function FindWord(ByVal strPattern As String, ByVal bytLength As Byte) As String
    Dim N As Long
    Dim lngStart As Long, lngEnd As Long
    If Left(strPattern, 1) = "*" Then
        lngStart = 0
        lngEnd = UBound(strWords)
    Else
        lngStart = lngLetterStartPositions(Asc(Left(strPattern, 1)) - Asc("a"))
        If Left(strPattern, 1) = "z" Then
            ' z has no letter after it, so we just use the end of the array
            lngEnd = UBound(strWords)
        Else
            ' check until we get to words starting with the next letter
            lngEnd = lngLetterStartPositions((Asc(Left(strPattern, 1)) - Asc("a")) + 1)
        End If
    End If
    Do
        N = RNG(lngStart, lngEnd)
        If Len(strWords(N)) <> bytLength Then
            N = -1
        Else
            If Not (strWords(N) Like strPattern) Then
                N = -1
            End If
        End If
    Loop Until N >= 0
    FindWord = strWords(N)
End Function

Private Function CheckSquare(ByVal strUserLetter As String, ByVal bytX As Byte, ByVal bytY As Byte) As Boolean
    If arrBattlefield(bytX, bytY).blnTree Then
        Status "Letters can't go over trees"
        Exit Function
    End If
    If arrBattlefield(bytX, bytY).blnCastleWall Then
        Status "Letters can't go over the castle wall"
        Exit Function
    End If
    If arrBattlefield(bytX, bytY).blnCastleMud Then
        Status "Letters can't go into the castle"
        Exit Function
    End If
    If arrBattlefield(bytX, bytY).blnWater Then
        Status "Letters can't go over water"
        Exit Function
    End If
    If arrBattlefield(bytX, bytY).blnDefenceWall Then
        If strUserLetter <> arrBattlefield(bytX, bytY).strLetter Then
            Status "Letters don't match up on connecting walls"
            Exit Function
        End If
    End If
    CheckSquare = True
End Function

Public Sub SetBattlefieldSize(ByVal bytWidth As Byte, ByVal bytHeight As Byte)
    With frmMain
        .Visible = False
        .ScaleMode = vbTwips
        .Width = ((bytWidth * 32) * Screen.TwipsPerPixelX) + (.Width - .ScaleWidth)
        .Height = ((bytHeight * 32) * Screen.TwipsPerPixelY) + (.Height - .ScaleHeight)
        .Left = (Screen.Width / 2) - (.Width / 2)
        .Top = (Screen.Height / 2) - (.Height / 2)
        .ScaleMode = vbPixels
        .Visible = True
    End With
End Sub

Public Function RNG(ByVal intMin As Long, ByVal intMax As Long, Optional blnEven As Boolean) As Long
    ' random number between from the min to max
    RNG = Int(((intMax + 1) - intMin) * Rnd) + intMin
    ' if it should be even, round it down
    If blnEven Then RNG = Int(RNG / 2) * 2
End Function
